<?php
define('SSC_APPLICATION_PATH', dirname(__FILE__));

abstract class Ssc_Application
{
    static protected $_namespaces = array(
        'controller' => array(
            'namespace' => 'Ssc_Controller',
            'folder'    => 'controllers',
        ),
        'model' => array(
            'namespace' => 'Ssc_Model',
            'folder'    => 'models',
        ),
        'service' => array(
            'namespace' => 'Ssc_Service',
            'folder'    => 'services',
        ),
        'form' => array(
            'namespace' => 'Ssc_Form',
            'folder'    => 'forms',
        ),
    );

    static protected $_singletons = array();

    static public function dispatch($controller='index', $action='index',
        $params = NULL)
    {
        if (empty($controller)) {
            $controller = 'index';
        } else {
            $controller = strtolower($controller);
        }
        if (empty($action)) {
            $action = 'index';
        } else {
            $action = strtolower($action);
        }

        $controllerClass = self::camelize($controller) . 'Controller';
        $controllerObject = self::factory($controllerClass, 'controller');

        $controllerObject->setParams((array) $params);

        $controllerObject->preDispatch($action);

        $actionName = self::camelize($action, FALSE);
        $method = $actionName . ('Action');
        $controllerObject->setViewScript($controller . '/' . $action . '.phtml');

        $result = $controllerObject->$method();

        $viewScript = $controllerObject->getViewScript();
        if (!empty($viewScript)) {
            /**
            * @var Ssc_View_Engine
            */
            $view = $controllerObject->getView();

            if (($layout = $view->layout)
                && ($layoutScript = $controllerObject->getLayoutScript())
            ) {
                $layout->content = $view->render($viewScript);
                $layout->renderDirect($layoutScript);
            } else {
                $view->renderDirect($viewScript);
            }
        }

        $controllerObject->postDispatch($action);

        return $result;
    }

    static public function getModel($name)
    {
        return self::factory($name, 'model', TRUE);
    }

    static public function getService($name)
    {
        return self::factory($name, 'service');
    }

    static public function getForm($name)
    {
        return self::factory($name, 'form');
    }

    static public function factory($name, $type, $singleton = FALSE)
    {
        $info = self::$_namespaces[$type];
        $class = $info['namespace'] . '_' . $name;

        if ($singleton && isset(self::$_singletons[$class])) {
            return self::$_singletons[$class];
        }

        require_once SSC_APPLICATION_PATH . '/' . $info['folder'] . '/'
            . str_replace('_', '/', $name) . '.php';
        $result = new $class;

        if ($singleton) {
            self::$_singletons[$class] = $result;
        }

        return $result;
    }

    static public function camelize($string, $first=TRUE)
    {
        $string = ucwords(str_replace('-', ' ', $string));
        $string = str_replace(' ', '', $string);
        if (!$first) {
            $string = strtolower($string[0]) . substr($string, 1);
        }

        return $string;
    }

    static public function url($action=NULL, $controller=NULL, $parameters=NULL)
    {
        $get = is_null($parameters)
            ? array()
            : $parameters;
        $get['page'] = Ssc_WpPlugin::SLUG;
        $get['action'] = $action;
        $get['controller'] = $controller;
        foreach ($get as $name=>$value) {
            if (is_null($value)) {
                unset($get[$name]);
            }
        }

        return dirname($_SERVER['SCRIPT_NAME']) . '/admin.php?' . http_build_query($get, '', '&');
    }
}